import { useObservable } from "@reactive-js/react";
import { map, scan } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
import { createElement, useMemo } from "react";
const empty = {
    path: "",
    query: "",
    fragment: "",
};
export const Router = function Router(props) {
    const { locationStore, notFound, routes, scheduler } = props;
    const observable = useMemo(() => {
        const routeMap = {};
        for (const [path, component] of routes) {
            routeMap[path] = component;
        }
        const uriUpdater = (updater) => {
            locationStore.dispatch(updater);
        };
        const pairify = ([_, oldState], next) => oldState === empty ? [undefined, next] : [oldState, next];
        return pipe(locationStore, scan(pairify, () => [
            undefined,
            empty,
        ]), map(([referer, uri]) => createElement(routeMap[uri.path] || notFound, {
            referer,
            uri,
            uriUpdater,
        })));
    }, [locationStore, notFound, routes]);
    const element = useObservable(observable, scheduler);
    return (element !== null && element !== void 0 ? element : null);
};
//# sourceMappingURL=router.js.map